/*
 * Experiment 1: Slidy Clock
 * Rob Miles January 2010: www.robmiles.com
 * 
 * Version 1.0
 * 
 * This program displays four layered textures on top of each other and moes them slowly in relation to each other to get a 
 * rather nice graphical effect. I think.
 * 
 * Feel free to experiment with different imaages, rates of movement and transparency.
 * 
 * Thanks to Raff for his Chumby template at http://www.aromatic.org/chumby/ChumbyDevelop.zip
 * 
 */

class ChumbyApp
{
	private var parent:MovieClip;
	private var menu:Menu;
	
	private var clockText:TextField;
	private var clockShadow:TextField;
	private var statusText:TextField;
	
	private var timerID:Number;
	private var activeImage:MovieClip;
	public var imageLoaded:Boolean = false;
	
	private var dX:Number = 1;
	private var dY:Number = 1;
	private var dAlpha:Number = -0.01;
	private var alpha:Number = 1;
	
	private var drawLayer:PictureLayer ;
	
	private var layers:Array = new Array(4);
	
	public var Message:String = "Starting";
	
	private function randRange(low:Number, high:Number)
	{
		return low + random(high - low);
	}
	
	private var TEXT_FORE:Number = 0xef1e23;
	private var TEXT_BACK:Number = 0xff00d8;
	
	public var DIAGNOSTICS:Boolean = false;
	
	function ChumbyApp(mc:MovieClip)
	{
		// save reference to main movieclip
		this.parent = mc;
		
		// make the back of the screen white
		Util.drawRect(mc, 0, 0, Util.SCREEN_WIDTH, Util.SCREEN_HEIGHT, Util.WHITE);
		
		// Build a set of layers
		
		for (var i:Number = 0; i < layers.length ; i++) 
		{
			activeImage = mc.attachMovie("library.MoodSquares.png", "activeImage", mc.getNextHighestDepth());
			
			// each row is larger than the one "behind" it
			
			var percentScale:Number = (100 + (i * 80)) / 100;
		
			// scale the image
			activeImage._xscale = 100 * percentScale;
			activeImage._yscale = 100 * percentScale;
			
			// The X value of the image will vary between this minium and 0
			var minX:Number = Util.SCREEN_WIDTH - (activeImage._width);
			var minY:Number = Util.SCREEN_HEIGHT - (activeImage._height);
			
			if ( DIAGNOSTICS) {
				trace("Layer:" + i + " Scale:" + percentScale + " Width:" + activeImage._width + " Height:" + activeImage._height);
			}
			
			drawLayer = new PictureLayer( 
				this,
				activeImage,
				minX, 0, // low and high values for x
				minY, 0, // low and high values for y
				randRange( -100, 100) / 50, // x speed
				randRange( -100, 100) / 50, // y speed
				randRange(minX, 0), // initial X position
				randRange(minY, 0)); // initial Y position
			
			layers[i] = drawLayer;
		}
		
		clockText = Util.getTextField(mc, 20, 70, 80, "00:00:00", 1);
		clockText.textColor = TEXT_FORE;
		clockShadow = Util.getTextField(mc, 23, 73, 80, "00:00:00", 1);
		clockShadow.textColor = TEXT_BACK;
		
		if ( DIAGNOSTICS ) 
		{
			statusText = Util.getTextField(mc, 20, 210, 20, "Starting", 1);
		}
		
		updateClock();
		
		timerID = setInterval(this, "updateTime", 50);
	}
	
	// Loads an image from a url. Updates the status display if DIAGNOSTICS is true
	// Returns the image loaded as a movie clip
	
	function downloadImage(url:String)
	{
		// Destination for the load 
		var container:MovieClip = parent.createEmptyMovieClip("container", parent.getNextHighestDepth());
		
		// Loader to fetch movie clip
		var loader:MovieClipLoader = new MovieClipLoader();
		
		// Object to be used to monitor progress
		var listener:Object = new Object();
		
		// Reference to application that we are running within
		var app:ChumbyApp = this;
		
		// Create function on listener object to generate load progress messate
		listener.onLoadProgress = function(target:MovieClip, bytesLoaded:Number, bytesTotal:Number):Void {
			if (app.DIAGNOSTICS) {
				trace(target + ".onLoadProgress with " + bytesLoaded + " bytes of " + bytesTotal);            
			}
		}
		
		// Create a function on listener object that is called when the load is completed
		listener.onLoadInit = function(mc:MovieClip) {
			trace("onLoadInit: " + mc);
			// Set the flag in the parent app to indicate the image has been loaded
			app.imageLoaded = true;
			
			mc._width = 80;
			mc._height = 60;
		}
		
		// Create a function to handle errors
		listener.onLoadError  = function(target_mc:MovieClip, errorCode:String):Void {
			trace("Load Error: " + errorCode);
		}
		
		// Add the listener object to the loader
		loader.addListener(listener);
		
		// Load the asset from the url and place it in the destination
		loader.loadClip(url, container);
		
		return container;
	}
	
	// Updates the display. Called 12 times a second
	
	function updateTime()
	{
		updateClock();
		
		for (var i:Number = 0; i < layers.length ; i++) 
		{
			var thisLayer:PictureLayer;
			thisLayer = layers[i];
			thisLayer.Update();
		}
		
	}
	
	private var lastTime:Number = -1;
	
	private function getStr(input:Number) {
		if ( input < 10) {
			return "0" + string(input);
		}
		return string(input);
	}
	
	function updateClock()
	{
		var now:Date = new Date();
		var n:Number = Math.round(now.getTime() / 1000);
		if ( n == lastTime) return;
		lastTime = n;
		var days = int(n / (60*60*24));
		n = n % (60*60*24);
		var hours = int(n / (60 * 60));
		n = n % (60*60);
		var mins = int(n / 60);
		var secs = int(n % 60);

		var timeString:String = getStr(hours) + ":" + getStr(mins) + ":" + getStr(secs);
		
		clockShadow.text = timeString;
		clockText.text = timeString;
		
		if(DIAGNOSTICS) {
		    statusText.text = Message;
			for (var i:Number = 0; i < layers.length ; i++) 
			{
				var thisLayer:PictureLayer;
				thisLayer = layers[i];
				trace ( "Layer: " + i + thisLayer.toString());
			}			
		}
	}
	
	static function main(mc:MovieClip)
	{
		var app = new ChumbyApp(mc);
	}
}
